package fr.inria.eventcloud.deployment.g5k;

import org.objectweb.proactive.api.PAActiveObject;
import org.objectweb.proactive.extensions.p2p.structured.deployment.gcmdeployment.GcmDeploymentNodeProvider;

import com.hp.hpl.jena.graph.Node;

import fr.inria.eventcloud.EventCloudDescription;
import fr.inria.eventcloud.EventCloudsRegistry;
import fr.inria.eventcloud.api.EventCloudId;
import fr.inria.eventcloud.api.PublishApi;
import fr.inria.eventcloud.api.Quadruple;
import fr.inria.eventcloud.api.SubscribeApi;
import fr.inria.eventcloud.api.Subscription;
import fr.inria.eventcloud.deployment.EventCloudDeployer;
import fr.inria.eventcloud.deployment.EventCloudDeploymentDescriptor;
import fr.inria.eventcloud.factories.EventCloudsRegistryFactory;
import fr.inria.eventcloud.factories.ProxyFactory;
import fr.inria.eventcloud.providers.SemanticInMemoryOverlayProvider;


public class Test {

    public static void main(String[] args) throws Exception {
        System.out.println("Test with 10 peers, 1 publisher and 1 subscriber");

        System.out.println();

        System.out.println("********** Starting deployment of the EventCloud **********");

        System.out.println();

        GcmDeploymentNodeProvider nodeProvider = new GcmDeploymentNodeProvider("resources/GCMA.xml");
        nodeProvider.start();

        EventCloudsRegistry registry = EventCloudsRegistryFactory.newEventCloudsRegistry(nodeProvider
                .getGcmVirtualNode("RegistryVN"));

        EventCloudId eventCloudId = new EventCloudId();
        EventCloudDescription eventCloudDescription = new EventCloudDescription(eventCloudId);
        EventCloudDeploymentDescriptor descriptor = new EventCloudDeploymentDescriptor(
            new SemanticInMemoryOverlayProvider());
        descriptor.setNodeProvider(nodeProvider);
        EventCloudDeployer deployer = new EventCloudDeployer(eventCloudDescription, descriptor);
        deployer.deploy(1, 10);
        registry.register(deployer);

        SubscribeApi subscribeProxy = ProxyFactory.newSubscribeProxy(
                nodeProvider.getGcmVirtualNode("ProxyVN"), PAActiveObject.getUrl(registry), eventCloudId);

        PublishApi publishProxy = ProxyFactory.newPublishProxy(nodeProvider.getGcmVirtualNode("ProxyVN"),
                PAActiveObject.getUrl(registry), eventCloudId);

        System.out.println();

        System.out.println("********** End of deployment **********");

        System.out.println();

        Subscription subscription = new Subscription(
            "PREFIX foaf: <http://xmlns.com/foaf/0.1/> SELECT ?name ?email ?g WHERE { GRAPH ?g { ?id foaf:name ?name . ?id foaf:email ?email } }");

        CustomBindingListener listener = PAActiveObject.newActive(CustomBindingListener.class, new Object[0]);
        subscribeProxy.subscribe(subscription, listener);

        System.out.println("Subscription with id " + subscription.getId() + " has been registered");

        Thread.sleep(2000);

        long publicationTime = System.currentTimeMillis();

        Quadruple q1 = new Quadruple(Node.createURI("https://plus.google.com/825349613"),
            Node.createURI("https://plus.google.com/107234124364605485774"),
            Node.createURI("http://xmlns.com/foaf/0.1/email"), Node.createLiteral("user1@company.com"));
        q1.setPublicationTime(publicationTime);
        publishProxy.publish(q1);

        Quadruple q2 = new Quadruple(Node.createURI("https://plus.google.com/825349613"),
            Node.createURI("https://plus.google.com/107234124364605485774"),
            Node.createURI("http://xmlns.com/foaf/0.1/name"), Node.createLiteral("User1"));
        q2.setPublicationTime(publicationTime);
        publishProxy.publish(q2);

        publicationTime = System.currentTimeMillis();

        Quadruple q3 = new Quadruple(
            Node.createURI("https://plus.google.com/3283940594/2011-2012-08-30-18:13:05"),
            Node.createURI("https://plus.google.com/107545688688906540962"),
            Node.createURI("http://xmlns.com/foaf/0.1/email"), Node.createLiteral("user2@company.com"));
        q3.setPublicationTime(publicationTime);
        publishProxy.publish(q3);

        Quadruple q4 = new Quadruple(
            Node.createURI("https://plus.google.com/3283940594/2011-2012-08-30-18:13:05"),
            Node.createURI("https://plus.google.com/107545688688906540962"),
            Node.createURI("http://xmlns.com/foaf/0.1/name"), Node.createLiteral("User 2"));
        q4.setPublicationTime(publicationTime);
        publishProxy.publish(q4);

        while (listener.getBindingsReceived() != 2) {
            Thread.sleep(1000);
        }

        nodeProvider.terminate();

        System.out.println();

        System.out.println("Success!!!");

        System.exit(0);
    }

}
